import java.awt.*;
import java.awt.event.*;
import java.util.*;
import javax.swing.*;

/**
  Clase cliente en la que creamos una instancia del marco
  en el que se dibujan bolas 
*/
public class Test extends JApplet{

   public void init(){
     Container c = getContentPane();
     c.add(new PanelConBotonesYBolas());
   }

   public static void main(String[] args) {
     JFrame frame = new JFrame();
     frame.setTitle("Marco con Bolas");

     // Obtener las dimensiones de la pantalla
     Toolkit kit = Toolkit.getDefaultToolkit();
     Dimension screenSize = kit.getScreenSize();
     int screenHeight = screenSize.height;
     int screenWidth = screenSize.width;

     // Centrar el marco en la pantalla
     frame.setSize(screenWidth/2, screenHeight/2);
     frame.setLocation(screenWidth/4, screenHeight/4);

     Container c = frame.getContentPane();
     c.add(new PanelConBotonesYBolas());
     
     frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
     frame.show();
  }

}

/**
  Marco con un lienzo y botones 
*/
class PanelConBotonesYBolas extends JPanel {
   private LienzoConBolas lienzo = null;

   /**
     Constructor de un marco que contine un LienzoConBolas y
     botones en empezar y cerrar.
   */
   public PanelConBotonesYBolas() {
      this.setLayout(new BorderLayout());

      lienzo = new LienzoConBolas();
      this.add(lienzo, BorderLayout.CENTER);

      JPanel bp = new JPanel();
      agregarBoton(bp, "Empezar",
         new ActionListener()
            {  
               public void actionPerformed(ActionEvent evt)
               {
                  agregarBola();
               }
            });

      agregarBoton(bp, "Cerrar",
         new ActionListener()
            {
               public void actionPerformed(ActionEvent evt)
               {
                  System.exit(0);
               }
            });
      this.add(bp, BorderLayout.SOUTH);
   }

   /**
      Aade un botn a un contenedor 
      @param c el contenedor 
      @param title ttulo del boton 
      @param listener clase adaptadora del botn 
   */
   public void agregarBoton(Container c, String title, ActionListener listener) {
      JButton b = new JButton(title);
      c.add(b);
      b.addActionListener(listener);
   }

   /**
     Aade una Bola al lienzo 
   */
   public void agregarBola() {
     Bola b = new Bola(lienzo);
     lienzo.agregar(b);
     /**
       * Esto no funciona, porque el thread que repinta nunca toma el control 
       * Es necesario implementarlo en un Thread
       */
     b.mover();
   }
}

/**
  Un lienzo sobre el que se dibujan bolas 
*/
class LienzoConBolas extends JPanel {
   private ArrayList bolas = new ArrayList();

   /**
      Aade una bola al lienzo 
      @param b la bola a aadir 
   */
   public void agregar(Bola b) {
      bolas.add(b);
   }

   public void paintComponent(Graphics g) {
      super.paintComponent(g);
      for (int i = 0; i < bolas.size(); i++) {
         Bola b = (Bola)bolas.get(i);
         b.dibujar(g);
      }     
   }
}

/**
   Una bola que se mueve y rebota en los bordes de la Componente
*/
class Bola {
   private Component lienzo;
   private static final int XSIZE = 15;
   private static final int YSIZE = 15;
   private int x = 0;
   private int y = 0;
   private int dx = 2;
   private int dy = 2;

   /**
      Crea una bola en la esquina superior izquierda 
      @c Componente sobre la cual rebota la bola 
   */
   public Bola(Component c) { 
     lienzo = c; 
   }

   /**
     Dibuja una bola en la correspondiente posicin x y 
      @param g el contexto grfico 
   */
   public void dibujar(Graphics g) {
      g.fillOval(x, y, XSIZE, YSIZE);
   }

   /**
      Mueve la bola a la siguiente posicin, 
      rebotando si se choca con el borde.
   */
   public void mover() {
      x += dx;
      y += dy;
      if (x < 0) { 
         x = 0;
         dx = -dx;
      }
      if (x + XSIZE >= lienzo.getWidth()) {
         x = lienzo.getWidth() - XSIZE; 
         dx = -dx; 
      }
      if (y < 0) {
         y = 0; 
         dy = -dy;
      }
      if (y + YSIZE >= lienzo.getHeight()) {
         y = lienzo.getHeight() - YSIZE;
         dy = -dy; 
      }
      lienzo.repaint();
   }
}
